@@ -2,17 +2,16 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 5 End-User License
-   Agreement and JUCE 5 Privacy Policy (both updated and effective as of the
-   27th April 2017).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-5-licence
-   Privacy Policy: www.juce.com/juce-5-privacy-policy
+   End User License Agreement: www.juce.com/juce-7-licence
+   Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
    www.gnu.org/licenses).
@@ -31,30 +30,32 @@ namespace dsp
 
 namespace SIMDRegister_test_internal
 {
-    template <typename type, typename = void> struct RandomPrimitive {};
-
     template <typename type>
-    struct RandomPrimitive<type, typename std::enable_if<std::is_floating_point<type>::value>::type>
+    struct RandomPrimitive
     {
         static type next (Random& random)
         {
-            return static_cast<type> (std::is_signed<type>::value ? (random.nextFloat() * 16.0) - 8.0
-                                                                  : (random.nextFloat() * 8.0));
-
+            if constexpr (std::is_floating_point_v<type>)
+            {
+                return static_cast<type> (std::is_signed_v<type> ? (random.nextFloat() * 16.0) - 8.0
+                                                                 : (random.nextFloat() * 8.0));
+            }
+            else if constexpr (std::is_integral_v<type>)
+            {
+                return static_cast<type> (random.nextInt64());
+            }
         }
     };
 
     template <typename type>
-    struct RandomPrimitive<type, typename std::enable_if<std::is_integral<type>::value>::type>
+    struct RandomValue
     {
         static type next (Random& random)
         {
-            return static_cast<type> (random.nextInt64());
-
+            return RandomPrimitive<type>::next (random);
         }
     };
 
-    template <typename type> struct RandomValue { static type next (Random& random) { return RandomPrimitive<type>::next (random); } };
     template <typename type>
     struct RandomValue<std::complex<type>>
     {
@@ -64,45 +65,14 @@ namespace SIMDRegister_test_internal
         }
     };
 
-
     template <typename type>
-    struct VecFiller
+    static void fillVec (type* dst, Random& random)
     {
-        static void fill (type* dst, const int size, Random& random)
+        std::generate_n (dst, SIMDRegister<type>::SIMDNumElements, [&]
         {
-            for (int i = 0; i < size; ++i)
-                dst[i] = RandomValue<type>::next (random);
-        }
-    };
-
-    // We need to specialise for complex types: otherwise GCC 6 gives
-    // us an ICE internal compiler error after which the compiler seg faults.
-    template <typename type>
-    struct VecFiller<std::complex<type>>
-    {
-        static void fill (std::complex<type>* dst, const int size, Random& random)
-        {
-            for (int i = 0; i < size; ++i)
-                dst[i] = std::complex<type> (RandomValue<type>::next (random), RandomValue<type>::next (random));
-        }
-    };
-
-    template <typename type>
-    struct VecFiller<SIMDRegister<type>>
-    {
-        static SIMDRegister<type> fill (Random& random)
-        {
-            constexpr int size = (int) SIMDRegister<type>::SIMDNumElements;
-           #ifdef _MSC_VER
-            __declspec(align(sizeof (SIMDRegister<type>))) type elements[size];
-           #else
-            type elements[(size_t) size] __attribute__((aligned(sizeof (SIMDRegister<type>))));
-           #endif
-
-            VecFiller<type>::fill (elements, size, random);
-            return SIMDRegister<type>::fromRawArray (elements);
-        }
-    };
+            return RandomValue<type>::next (random);
+        });
+    }
 
     // Avoid visual studio warning
     template <typename type>
@@ -128,7 +98,7 @@ namespace SIMDRegister_test_internal
     {
         return difference (a - b);
     }
-}
+} // namespace SIMDRegister_test_internal
 
 // These tests need to be strictly run on all platforms supported by JUCE as the
 // SIMD code is highly platform dependent.
@@ -136,6 +106,8 @@ namespace SIMDRegister_test_internal
 class SIMDRegisterUnitTests   : public UnitTest
 {
 public:
+    template <typename> struct Tag {};
+
     SIMDRegisterUnitTests()
         : UnitTest ("SIMDRegister UnitTests", UnitTestCategories::dsp)
     {}
@@ -145,19 +117,12 @@ public:
     template <typename type>
     static bool allValuesEqualTo (const SIMDRegister<type>& vec, const type scalar)
     {
-       #ifdef _MSC_VER
-        __declspec(align(sizeof (SIMDRegister<type>))) type elements[SIMDRegister<type>::SIMDNumElements];
-       #else
-        type elements[SIMDRegister<type>::SIMDNumElements] __attribute__((aligned(sizeof (SIMDRegister<type>))));
-       #endif
+        alignas (sizeof (SIMDRegister<type>)) type elements[SIMDRegister<type>::SIMDNumElements];
 
         vec.copyToRawArray (elements);
 
         // as we do not want to rely on the access operator we cast this to a primitive pointer
-        for (size_t i = 0; i < SIMDRegister<type>::SIMDNumElements; ++i)
-            if (elements[i] != scalar) return false;
-
-        return true;
+        return std::all_of (std::begin (elements), std::end (elements), [scalar] (const auto x) { return exactlyEqual (x, scalar); });
     }
 
     template <typename type>
@@ -195,7 +160,7 @@ public:
     }
 
     //==============================================================================
-    // Someuseful operations to test
+    // Some useful operations to test
     struct Addition
     {
         template <typename typeOne, typename typeTwo>
@@ -291,7 +256,7 @@ public:
     struct InitializationTest
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             u.expect (allValuesEqualTo<type> (SIMDRegister<type>::expand (static_cast<type> (23)), 23));
 
@@ -301,7 +266,7 @@ public:
                #else
                 type elements[SIMDRegister<type>::SIMDNumElements] __attribute__((aligned(sizeof (SIMDRegister<type>))));
                #endif
-                SIMDRegister_test_internal::VecFiller<type>::fill (elements, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (elements, random);
                 SIMDRegister<type> a (SIMDRegister<type>::fromRawArray (elements));
 
                 u.expect (vecEqualToArray (a, elements));
@@ -317,13 +282,13 @@ public:
     struct AccessTest
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             // set-up
             SIMDRegister<type> a;
             type array [SIMDRegister<type>::SIMDNumElements];
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (array, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (array, random);
 
             // Test non-const access operator
             for (size_t i = 0; i < SIMDRegister<type>::SIMDNumElements; ++i)
@@ -335,7 +300,7 @@ public:
             const SIMDRegister<type>& b = a;
 
             for (size_t i = 0; i < SIMDRegister<type>::SIMDNumElements; ++i)
-                u.expect (b[i] == array[i]);
+                u.expect (exactlyEqual (b[i], array[i]));
         }
     };
 
@@ -343,7 +308,7 @@ public:
     struct OperatorTests
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             for (int n = 0; n < 100; ++n)
             {
@@ -356,9 +321,9 @@ public:
                 type array_b [SIMDRegister<type>::SIMDNumElements];
                 type array_c [SIMDRegister<type>::SIMDNumElements];
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (array_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (array_c, random);
 
                 copy (a, array_a); copy (b, array_b); copy (c, array_c);
 
@@ -371,9 +336,9 @@ public:
                 u.expect (vecEqualToArray (a, array_a));
                 u.expect (vecEqualToArray (b, array_b));
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (array_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (array_c, random);
 
                 copy (a, array_a); copy (b, array_b); copy (c, array_c);
 
@@ -387,9 +352,9 @@ public:
                 u.expect (vecEqualToArray (b, array_b));
 
                 // set-up again
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (array_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (array_c, random);
                 copy (a, array_a); copy (b, array_b); copy (c, array_c);
 
                 // test out-of-place with both params being vectors
@@ -419,7 +384,7 @@ public:
     struct BitOperatorTests
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             typedef typename SIMDRegister<type>::vMaskType vMaskType;
             typedef typename SIMDRegister<type>::MaskType MaskType;
@@ -439,7 +404,7 @@ public:
                     } a, b;
 
                     vMaskType bitmask = vMaskType::expand (static_cast<MaskType> (1) << (sizeof (MaskType) - 1));
-                    SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
+                    SIMDRegister_test_internal::fillVec (array_a, random);
                     copy (a.floatVersion, array_a);
                     copy (b.floatVersion, array_a);
 
@@ -467,9 +432,9 @@ public:
                 type float_a [SIMDRegister<type>::SIMDNumElements];
                 type float_c [SIMDRegister<type>::SIMDNumElements];
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<MaskType>::fill (array_b, SIMDRegister<MaskType>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (float_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (float_c, random);
 
                 memcpy (array_a, float_a, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
                 memcpy (array_c, float_c, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
@@ -485,9 +450,9 @@ public:
                 u.expect (vecEqualToArray (a, float_a));
                 u.expect (vecEqualToArray (b, array_b));
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<MaskType>::fill (array_b, SIMDRegister<MaskType>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (float_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (float_c, random);
                 memcpy (array_a, float_a, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
                 memcpy (array_c, float_c, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
                 copy (a, float_a); copy (b, array_b); copy (c, float_c);
@@ -503,9 +468,9 @@ public:
                 u.expect (vecEqualToArray (b, array_b));
 
                 // set-up again
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<MaskType>::fill (array_b, SIMDRegister<MaskType>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (float_c, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (float_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
+                SIMDRegister_test_internal::fillVec (float_c, random);
                 memcpy (array_a, float_a, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
                 memcpy (array_c, float_c, sizeof (type) * SIMDRegister<type>::SIMDNumElements);
                 copy (a, float_a); copy (b, array_b); copy (c, float_c);
@@ -543,7 +508,7 @@ public:
     struct CheckComparisonOps
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             typedef typename SIMDRegister<type>::vMaskType vMaskType;
             typedef typename SIMDRegister<type>::MaskType MaskType;
@@ -561,14 +526,14 @@ public:
                 MaskType array_ge  [SIMDRegister<type>::SIMDNumElements];
 
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (array_a, random);
+                SIMDRegister_test_internal::fillVec (array_b, random);
 
                 // do check
                 for (size_t j = 0; j < SIMDRegister<type>::SIMDNumElements; ++j)
                 {
-                    array_eq  [j] = (array_a[j] == array_b[j]) ? static_cast<MaskType> (-1) : 0;
-                    array_neq [j] = (array_a[j] != array_b[j]) ? static_cast<MaskType> (-1) : 0;
+                    array_eq  [j] = (  exactlyEqual (array_a[j], array_b[j])) ? static_cast<MaskType> (-1) : 0;
+                    array_neq [j] = (! exactlyEqual (array_a[j], array_b[j])) ? static_cast<MaskType> (-1) : 0;
                     array_lt  [j] = (array_a[j] <  array_b[j]) ? static_cast<MaskType> (-1) : 0;
                     array_le  [j] = (array_a[j] <= array_b[j]) ? static_cast<MaskType> (-1) : 0;
                     array_gt  [j] = (array_a[j] >  array_b[j]) ? static_cast<MaskType> (-1) : 0;
@@ -599,8 +564,8 @@ public:
 
                 do
                 {
-                    SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-                    SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
+                    SIMDRegister_test_internal::fillVec (array_a, random);
+                    SIMDRegister_test_internal::fillVec (array_b, random);
                 } while (std::equal (array_a, array_a + SIMDRegister<type>::SIMDNumElements, array_b));
 
                 copy (a, array_a);
@@ -610,7 +575,7 @@ public:
                 u.expect (! (a == b));
                 u.expect (! (b == a));
 
-                SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
+                SIMDRegister_test_internal::fillVec (array_a, random);
                 copy (a, array_a);
                 copy (b, array_a);
 
@@ -636,7 +601,7 @@ public:
     struct CheckMultiplyAdd
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             // set-up
             type array_a [SIMDRegister<type>::SIMDNumElements];
@@ -644,10 +609,10 @@ public:
             type array_c [SIMDRegister<type>::SIMDNumElements];
             type array_d [SIMDRegister<type>::SIMDNumElements];
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (array_a, SIMDRegister<type>::SIMDNumElements, random);
-            SIMDRegister_test_internal::VecFiller<type>::fill (array_b, SIMDRegister<type>::SIMDNumElements, random);
-            SIMDRegister_test_internal::VecFiller<type>::fill (array_c, SIMDRegister<type>::SIMDNumElements, random);
-            SIMDRegister_test_internal::VecFiller<type>::fill (array_d, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (array_a, random);
+            SIMDRegister_test_internal::fillVec (array_b, random);
+            SIMDRegister_test_internal::fillVec (array_c, random);
+            SIMDRegister_test_internal::fillVec (array_d, random);
 
             // check
             for (size_t i = 0; i < SIMDRegister<type>::SIMDNumElements; ++i)
@@ -668,7 +633,7 @@ public:
     struct CheckMinMax
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             for (int i = 0; i < 100; ++i)
             {
@@ -718,17 +683,14 @@ public:
     struct CheckSum
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             type array [SIMDRegister<type>::SIMDNumElements];
-            type sumCheck = 0;
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (array, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (array, random);
 
-            for (size_t j = 0; j < SIMDRegister<type>::SIMDNumElements; ++j)
-            {
-                sumCheck += array[j];
-            }
+            using AddedType = decltype (type{} + type{});
+            const auto sumCheck = (type) std::accumulate (std::begin (array), std::end (array), AddedType{});
 
             SIMDRegister<type> a;
             copy (a, array);
@@ -740,18 +702,18 @@ public:
     struct CheckAbs
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             type inArray[SIMDRegister<type>::SIMDNumElements];
             type outArray[SIMDRegister<type>::SIMDNumElements];
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (inArray, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (inArray, random);
 
             SIMDRegister<type> a;
             copy (a, inArray);
             a = SIMDRegister<type>::abs (a);
 
-            auto calcAbs = [] (type x) -> type { return x >= type (0) ? x : -x; };
+            auto calcAbs = [] (type x) -> type { return x >= type (0) ? x : type (-x); };
 
             for (size_t j = 0; j < SIMDRegister<type>::SIMDNumElements; ++j)
                 outArray[j] = calcAbs (inArray[j]);
@@ -763,12 +725,12 @@ public:
     struct CheckTruncate
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
             type inArray[SIMDRegister<type>::SIMDNumElements];
             type outArray[SIMDRegister<type>::SIMDNumElements];
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (inArray, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (inArray, random);
 
             SIMDRegister<type> a;
             copy (a, inArray);
@@ -784,13 +746,12 @@ public:
     struct CheckBoolEquals
     {
         template <typename type>
-        static void run (UnitTest& u, Random& random)
+        static void run (UnitTest& u, Random& random, Tag<type>)
         {
-            bool is_signed = std::is_signed<type>::value;
             type array [SIMDRegister<type>::SIMDNumElements];
 
-            auto value = is_signed ? static_cast<type> ((random.nextFloat() * 16.0) - 8.0)
-                                   : static_cast<type> (random.nextFloat() * 8.0);
+            auto value = std::is_signed_v<type> ? static_cast<type> ((random.nextFloat() * 16.0) - 8.0)
+                                                : static_cast<type> (random.nextFloat() * 8.0);
 
             std::fill (array, array + SIMDRegister<type>::SIMDNumElements, value);
             SIMDRegister<type> a, b;
@@ -803,14 +764,14 @@ public:
             u.expect (a != value);
             u.expect (! (a == value));
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (array, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (array, random);
             copy (a, array);
             copy (b, array);
 
             u.expect (a == b);
             u.expect (! (a != b));
 
-            SIMDRegister_test_internal::VecFiller<type>::fill (array, SIMDRegister<type>::SIMDNumElements, random);
+            SIMDRegister_test_internal::fillVec (array, random);
             copy (b, array);
 
             u.expect (a != b);
@@ -820,96 +781,96 @@ public:
 
     //==============================================================================
     template <class TheTest>
-    void runTestFloatingPoint (const char* unitTestName)
+    void runTestFloatingPoint (const char* unitTestName, TheTest)
     {
         beginTest (unitTestName);
 
         Random random = getRandom();
 
-        TheTest::template run<float>  (*this, random);
-        TheTest::template run<double> (*this, random);
+        TheTest::run (*this, random, Tag<float> {});
+        TheTest::run (*this, random, Tag<double>{});
     }
 
     //==============================================================================
     template <class TheTest>
-    void runTestForAllTypes (const char* unitTestName)
+    void runTestForAllTypes (const char* unitTestName, TheTest)
     {
         beginTest (unitTestName);
 
         Random random = getRandom();
 
-        TheTest::template run<float>   (*this, random);
-        TheTest::template run<double>  (*this, random);
-        TheTest::template run<int8_t>  (*this, random);
-        TheTest::template run<uint8_t> (*this, random);
-        TheTest::template run<int16_t> (*this, random);
-        TheTest::template run<uint16_t>(*this, random);
-        TheTest::template run<int32_t> (*this, random);
-        TheTest::template run<uint32_t>(*this, random);
-        TheTest::template run<int64_t> (*this, random);
-        TheTest::template run<uint64_t>(*this, random);
-        TheTest::template run<std::complex<float>>   (*this, random);
-        TheTest::template run<std::complex<double>>  (*this, random);
+        TheTest::run (*this, random, Tag<float>               {});
+        TheTest::run (*this, random, Tag<double>              {});
+        TheTest::run (*this, random, Tag<int8_t>              {});
+        TheTest::run (*this, random, Tag<uint8_t>             {});
+        TheTest::run (*this, random, Tag<int16_t>             {});
+        TheTest::run (*this, random, Tag<uint16_t>            {});
+        TheTest::run (*this, random, Tag<int32_t>             {});
+        TheTest::run (*this, random, Tag<uint32_t>            {});
+        TheTest::run (*this, random, Tag<int64_t>             {});
+        TheTest::run (*this, random, Tag<uint64_t>            {});
+        TheTest::run (*this, random, Tag<std::complex<float>> {});
+        TheTest::run (*this, random, Tag<std::complex<double>>{});
     }
 
     template <class TheTest>
-    void runTestNonComplex (const char* unitTestName)
+    void runTestNonComplex (const char* unitTestName, TheTest)
     {
         beginTest (unitTestName);
 
         Random random = getRandom();
 
-        TheTest::template run<float>   (*this, random);
-        TheTest::template run<double>  (*this, random);
-        TheTest::template run<int8_t>  (*this, random);
-        TheTest::template run<uint8_t> (*this, random);
-        TheTest::template run<int16_t> (*this, random);
-        TheTest::template run<uint16_t>(*this, random);
-        TheTest::template run<int32_t> (*this, random);
-        TheTest::template run<uint32_t>(*this, random);
-        TheTest::template run<int64_t> (*this, random);
-        TheTest::template run<uint64_t>(*this, random);
+        TheTest::run (*this, random, Tag<float>   {});
+        TheTest::run (*this, random, Tag<double>  {});
+        TheTest::run (*this, random, Tag<int8_t>  {});
+        TheTest::run (*this, random, Tag<uint8_t> {});
+        TheTest::run (*this, random, Tag<int16_t> {});
+        TheTest::run (*this, random, Tag<uint16_t>{});
+        TheTest::run (*this, random, Tag<int32_t> {});
+        TheTest::run (*this, random, Tag<uint32_t>{});
+        TheTest::run (*this, random, Tag<int64_t> {});
+        TheTest::run (*this, random, Tag<uint64_t>{});
     }
 
     template <class TheTest>
-    void runTestSigned (const char* unitTestName)
+    void runTestSigned (const char* unitTestName, TheTest)
     {
         beginTest (unitTestName);
 
         Random random = getRandom();
 
-        TheTest::template run<float>   (*this, random);
-        TheTest::template run<double>  (*this, random);
-        TheTest::template run<int8_t>  (*this, random);
-        TheTest::template run<int16_t> (*this, random);
-        TheTest::template run<int32_t> (*this, random);
-        TheTest::template run<int64_t> (*this, random);
+        TheTest::run (*this, random, Tag<float>  {});
+        TheTest::run (*this, random, Tag<double> {});
+        TheTest::run (*this, random, Tag<int8_t> {});
+        TheTest::run (*this, random, Tag<int16_t>{});
+        TheTest::run (*this, random, Tag<int32_t>{});
+        TheTest::run (*this, random, Tag<int64_t>{});
     }
 
     void runTest()
     {
-        runTestForAllTypes<InitializationTest> ("InitializationTest");
+        runTestForAllTypes ("InitializationTest", InitializationTest{});
 
-        runTestForAllTypes<AccessTest> ("AccessTest");
+        runTestForAllTypes ("AccessTest", AccessTest{});
 
-        runTestForAllTypes<OperatorTests<Addition>> ("AdditionOperators");
-        runTestForAllTypes<OperatorTests<Subtraction>> ("SubtractionOperators");
-        runTestForAllTypes<OperatorTests<Multiplication>> ("MultiplicationOperators");
+        runTestForAllTypes ("AdditionOperators", OperatorTests<Addition>{});
+        runTestForAllTypes ("SubtractionOperators", OperatorTests<Subtraction>{});
+        runTestForAllTypes ("MultiplicationOperators", OperatorTests<Multiplication>{});
 
-        runTestForAllTypes<BitOperatorTests<BitAND>> ("BitANDOperators");
-        runTestForAllTypes<BitOperatorTests<BitOR>>  ("BitOROperators");
-        runTestForAllTypes<BitOperatorTests<BitXOR>> ("BitXOROperators");
+        runTestForAllTypes ("BitANDOperators", BitOperatorTests<BitAND>{});
+        runTestForAllTypes ("BitOROperators", BitOperatorTests<BitOR>{});
+        runTestForAllTypes ("BitXOROperators", BitOperatorTests<BitXOR>{});
 
-        runTestNonComplex<CheckComparisonOps> ("CheckComparisons");
-        runTestNonComplex<CheckBoolEquals> ("CheckBoolEquals");
-        runTestNonComplex<CheckMinMax> ("CheckMinMax");
+        runTestNonComplex ("CheckComparisons", CheckComparisonOps{});
+        runTestNonComplex ("CheckBoolEquals", CheckBoolEquals{});
+        runTestNonComplex ("CheckMinMax", CheckMinMax{});
 
-        runTestForAllTypes<CheckMultiplyAdd> ("CheckMultiplyAdd");
-        runTestForAllTypes<CheckSum> ("CheckSum");
+        runTestForAllTypes ("CheckMultiplyAdd", CheckMultiplyAdd{});
+        runTestForAllTypes ("CheckSum", CheckSum{});
 
-        runTestSigned<CheckAbs> ("CheckAbs");
+        runTestSigned ("CheckAbs", CheckAbs{});
 
-        runTestFloatingPoint<CheckTruncate> ("CheckTruncate");
+        runTestFloatingPoint ("CheckTruncate", CheckTruncate{});
     }
 };
 
