@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -32,12 +32,12 @@
 
   ID:                 juce_events
   vendor:             juce
-  version:            6.1.6
+  version:            7.0.7
   name:               JUCE message and event handling classes
   description:        Classes for running an application's main event loop and sending/receiving messages, timers, etc.
   website:            http://www.juce.com/juce
   license:            ISC
-  minimumCppStandard: 14
+  minimumCppStandard: 17
 
   dependencies:       juce_core
 
@@ -82,6 +82,7 @@
 #include "broadcasters/juce_ActionBroadcaster.h"
 #include "broadcasters/juce_ActionListener.h"
 #include "broadcasters/juce_AsyncUpdater.h"
+#include "broadcasters/juce_LockingAsyncUpdater.h"
 #include "broadcasters/juce_ChangeListener.h"
 #include "broadcasters/juce_ChangeBroadcaster.h"
 #include "timers/juce_Timer.h"
@@ -93,14 +94,14 @@
 #include "native/juce_ScopedLowPowerModeDisabler.h"
 
 #if JUCE_LINUX || JUCE_BSD
- #include "native/juce_linux_EventLoop.h"
+ #include "native/juce_EventLoop_linux.h"
 #endif
 
 #if JUCE_WINDOWS
  #if JUCE_EVENTS_INCLUDE_WIN32_MESSAGE_WINDOW
-  #include "native/juce_win32_HiddenMessageWindow.h"
+  #include "native/juce_HiddenMessageWindow_windows.h"
  #endif
  #if JUCE_EVENTS_INCLUDE_WINRT_WRAPPER
-  #include "native/juce_win32_WinRTWrapper.h"
+  #include "native/juce_WinRTWrapper_windows.h"
  #endif
 #endif
