@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -28,11 +28,6 @@ class ThreadPoolJob;
 class ActionListener;
 class ActionBroadcaster;
 
-//==============================================================================
-#if JUCE_MODULE_AVAILABLE_juce_opengl
-class OpenGLContext;
-#endif
-
 //==============================================================================
 /** See MessageManager::callFunctionOnMessageThread() for use of this function type. */
 using MessageCallbackFunction = void* (void* userData);
@@ -84,7 +79,7 @@ public:
     */
     bool hasStopMessageBeenSent() const noexcept        { return quitMessagePosted.get() != 0; }
 
-   #if JUCE_MODAL_LOOPS_PERMITTED || DOXYGEN
+   #if JUCE_MODAL_LOOPS_PERMITTED
     /** Synchronously dispatches messages until a given time has elapsed.
 
         Returns false if a quit message has been posted by a call to stopDispatchLoop(),
@@ -208,7 +203,7 @@ public:
             Creates a new critical section to exclusively access methods which can
             only be called when the message manager is locked.
 
-            Unlike CrititcalSection, multiple instances of this lock class provide
+            Unlike CriticalSection, multiple instances of this lock class provide
             exclusive access to a single resource - the MessageManager.
         */
         Lock();
@@ -223,14 +218,14 @@ public:
             If another thread is currently using the MessageManager, this will wait until that
             thread releases the lock to the MessageManager.
 
-            This call will only exit if the lock was accquired by this thread. Calling abort while
+            This call will only exit if the lock was acquired by this thread. Calling abort while
             a thread is waiting for enter to finish, will have no effect.
 
             @see exit, abort
          */
          void enter() const noexcept;
 
-         /** Attempts to lock the meesage manager and exits if abort is called.
+         /** Attempts to lock the message manager and exits if abort is called.
 
             This method behaves identically to enter, except that it will abort waiting for
             the lock if the abort method is called.
@@ -270,7 +265,7 @@ public:
                  messageManagerLock.abort();
             }
 
-            @returns false if waiting for a lock was aborted, true if the lock was accquired.
+            @returns false if waiting for a lock was aborted, true if the lock was acquired.
             @see enter, abort, ScopedTryLock
         */
         bool tryEnter() const noexcept;
@@ -283,7 +278,7 @@ public:
         /** Unblocks a thread which is waiting in tryEnter
             Call this method if you want to unblock a thread which is waiting for the
             MessageManager lock in tryEnter.
-            This method does not have any effetc on a thread waiting for a lock in enter.
+            This method does not have any effect on a thread waiting for a lock in enter.
             @see tryEnter
         */
         void abort() const noexcept;
@@ -303,12 +298,12 @@ public:
         friend class ReferenceCountedObjectPtr<BlockingMessage>;
 
         bool tryAcquire (bool) const noexcept;
-        void messageCallback() const;
 
         //==============================================================================
         mutable ReferenceCountedObjectPtr<BlockingMessage> blockingMessage;
-        WaitableEvent lockedEvent;
-        mutable Atomic<int> abortWait, lockGained;
+        mutable std::mutex mutex;
+        mutable std::condition_variable condvar;
+        mutable bool abortWait = false, acquired = false;
     };
 
     //==============================================================================
@@ -333,12 +328,12 @@ private:
     Atomic<int> quitMessagePosted { 0 }, quitMessageReceived { 0 };
     Thread::ThreadID messageThreadId;
     Atomic<Thread::ThreadID> threadWithLock;
+    mutable std::mutex messageThreadIdMutex;
 
     static bool postMessageToSystemQueue (MessageBase*);
     static void* exitModalLoopCallback (void*);
     static void doPlatformSpecificInitialisation();
     static void doPlatformSpecificShutdown();
-    static bool dispatchNextMessageOnSystemQueue (bool returnIfNoPendingMessages);
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MessageManager)
 };
