@@ -2,15 +2,15 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2020 - Raw Material Software Limited
+   Copyright (c) 2022 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
 
-   By using JUCE, you agree to the terms of both the JUCE 6 End-User License
-   Agreement and JUCE Privacy Policy (both effective as of the 16th June 2020).
+   By using JUCE, you agree to the terms of both the JUCE 7 End-User License
+   Agreement and JUCE Privacy Policy.
 
-   End User License Agreement: www.juce.com/juce-6-licence
+   End User License Agreement: www.juce.com/juce-7-licence
    Privacy Policy: www.juce.com/juce-privacy-policy
 
    Or: You may also use this code under the terms of the GPL v3 (see
@@ -33,18 +33,20 @@
 
  BEGIN_JUCE_MODULE_DECLARATION
 
-  ID:                 juce_gui_extra
-  vendor:             juce
-  version:            6.1.6
-  name:               JUCE extended GUI classes
-  description:        Miscellaneous GUI classes for specialised tasks.
-  website:            http://www.juce.com/juce
-  license:            GPL/Commercial
-  minimumCppStandard: 14
+  ID:                     juce_gui_extra
+  vendor:                 juce
+  version:                7.0.7
+  name:                   JUCE extended GUI classes
+  description:            Miscellaneous GUI classes for specialised tasks.
+  website:                http://www.juce.com/juce
+  license:                GPL/Commercial
+  minimumCppStandard:     17
 
-  dependencies:       juce_gui_basics
-  OSXFrameworks:      WebKit
-  iOSFrameworks:      WebKit
+  dependencies:           juce_gui_basics
+  OSXFrameworks:          WebKit
+  iOSFrameworks:          WebKit
+  WeakiOSFrameworks:      UserNotifications
+  WeakMacOSFrameworks:    UserNotifications
 
  END_JUCE_MODULE_DECLARATION
 
