@@ -58,7 +58,7 @@ struct OnlineUnlockForm::OverlayComp  : public Component,
             cancelButton->addListener (this);
         }
 
-        startThread (4);
+        startThread (Priority::normal);
     }
 
     ~OverlayComp() override
@@ -100,15 +100,17 @@ struct OnlineUnlockForm::OverlayComp  : public Component,
 
         if (result.errorMessage.isNotEmpty())
         {
-            AlertWindow::showMessageBoxAsync (MessageBoxIconType::WarningIcon,
-                                              TRANS("Registration Failed"),
-                                              result.errorMessage);
+            auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::WarningIcon,
+                                                             TRANS ("Registration Failed"),
+                                                             result.errorMessage);
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
         }
         else if (result.informativeMessage.isNotEmpty())
         {
-            AlertWindow::showMessageBoxAsync (MessageBoxIconType::InfoIcon,
-                                              TRANS("Registration Complete!"),
-                                              result.informativeMessage);
+            auto options = MessageBoxOptions::makeOptionsOk (MessageBoxIconType::InfoIcon,
+                                                             TRANS ("Registration Complete!"),
+                                                             result.informativeMessage);
+            messageBox = AlertWindow::showScopedAsync (options, nullptr);
         }
         else if (result.urlToLaunch.isNotEmpty())
         {
@@ -143,6 +145,7 @@ struct OnlineUnlockForm::OverlayComp  : public Component,
     Spinner spinner;
     OnlineUnlockStatus::UnlockResult result;
     String email, password;
+    ScopedMessageBox messageBox;
 
     std::unique_ptr<TextButton> cancelButton;
 
